from __future__ import annotations

from collections import ChainMap
from pathlib import Path, PureWindowsPath
from typing import ClassVar, Mapping, Sequence

import arrow
import attr

from wap import __version__, log
from wap.config import TocConfig
from wap.exception import TocException
from wap.wowversion import WoWVersion

# tag names from https://wowwiki-archive.fandom.com/wiki/TOC_format
_OFFICIAL_TAGS = {
    "Interface",
    "Title",
    "Author",
    "Version",
    "Notes",
    "RequiredDeps",  # according to docs, this and the next have the same meaning to wow
    "Dependencies",
    "OptionalDeps",
    "LoadOnDemand",
    "LoadWith",
    "LoadManagers",
    "SavedVariables",
    "DefaultState",
    "Secure",
}

# we will warn about tags not in _OFFICIAL_TAGS that don't have the right prefix
_METADATA_TAG_PREFIX = "X-"


@attr.s(kw_only=True, auto_attribs=True, order=False, frozen=True)
class Toc:
    tags: Mapping[str, str]
    files: Sequence[Path]

    @classmethod
    def from_toc_config(cls, toc_config: TocConfig) -> Toc:
        return cls(
            tags=toc_config.tags,
            files=[Path(f) for f in toc_config.files],
        )

    def write(
        self,
        path: Path,
        addon_version: str,
        wow_version: WoWVersion,
    ) -> None:
        for file in self.files:
            rooted_file = path.parent / file
            if not rooted_file.is_file():
                raise TocException(
                    f"TOC file {path} lists {file}, but it does not exist"
                )

        extra_wap_tags = {
            "Interface": wow_version.interface_version(),
            "Version": addon_version,
        }

        tag_map = ChainMap(self.tags, extra_wap_tags)

        for key, value in extra_wap_tags.items():
            if key in self.tags:
                log.warn(
                    f"Overwriting wap-provided tag {key}={value} with {self.tags[key]}"
                )

        for key, value in self.tags.items():
            if key not in _OFFICIAL_TAGS and not key.startswith(_METADATA_TAG_PREFIX):
                log.warn(
                    f"TOC user-specified tag {key} does not have "
                    f"{_METADATA_TAG_PREFIX} prefix"
                )

        # TOC file use windows path separators
        windows_files = [PureWindowsPath(f) for f in self.files]

        lines = [
            f"# Generated by wap v{__version__} at {arrow.now().isoformat()}\n",
            "\n",
            *[f"## {key}: {value}\n" for key, value in tag_map.items()],
            "\n",
            *[f"{str(file)}\n" for file in windows_files],
        ]

        with path.open("w") as toc_file:
            for line in lines:
                toc_file.write(line)



import json
from pathlib import Path

import click

from wap import addon
from wap.commands.common import (
    addon_version_option,
    config_path_option,
    json_option,
    output_path_option,
)
from wap.config import Config


@click.command()
@config_path_option()
@output_path_option()
@addon_version_option()
@json_option()
def build(
    config_path: Path,
    output_path: Path,
    addon_version: str,
    show_json: bool,
) -> None:
    """
    Builds packages, creating a directory with your packaged addon and a zip archive.
    Outputs a JSON object describing the locations of the built files.
    """
    config = Config.from_path(config_path)

    output_map = {}

    for wow_version in config.wow_versions:
        build_path = addon.build_addon(
            config_path=config_path,
            addon_name=config.name,
            dir_configs=config.dir_configs,
            output_path=output_path,
            addon_version=addon_version,
            wow_version=wow_version,
        )

        zip_path = addon.zip_addon(
            addon_name=config.name,
            wow_version=wow_version,
            build_path=build_path,
        )

        output_map[wow_version.type()] = {
            "build_dir_path": str(build_path),
            "zip_file_path": str(zip_path),
        }

    if show_json:
        click.echo(json.dumps(output_map, indent=2))



from __future__ import annotations

import json
from pathlib import Path
from typing import Optional

import click

from wap import addon
from wap.commands.common import (
    WAP_WOW_ADDONS_PATH_ENVVAR_NAME,
    addon_version_option,
    config_path_option,
    json_option,
    output_path_option,
)
from wap.config import Config
from wap.exception import DevInstallException
from wap.wowversion import WoWVersion


class WowAddonsPathType(click.ParamType):
    name = "WOW_ADDONS_PATH"

    def convert(
        self,
        value: str,
        param: Optional[click.Parameter],
        ctx: Optional[click.Context],
    ) -> Path:
        path = Path(value)
        if not path.is_dir():
            self.fail(f"WoW AddOns path {value} is not a directory", param, ctx)

        not_addons_dir_exc_str = (
            f"WoW AddOns path {path} does not look like a WoW addons directory"
        )

        try:
            *_, wow_part, type_part, interface_part, addons_part = path.parts
        except ValueError:
            self.fail(not_addons_dir_exc_str, param, ctx)

        if (
            wow_part != "World of Warcraft"
            or interface_part != "Interface"
            or addons_part != "AddOns"
        ):
            self.fail(not_addons_dir_exc_str, param, ctx)

        if type_part not in WoWVersion.ADDONS_PATH_TYPE_MAP.keys():
            self.fail(not_addons_dir_exc_str, param, ctx)

        return path


WOW_ADDONS_PATH_TYPE = WowAddonsPathType()


@click.command()
@config_path_option()
@output_path_option()
@addon_version_option()
@json_option()
@click.option(
    "-w",
    "--wow-addons-path",
    envvar=WAP_WOW_ADDONS_PATH_ENVVAR_NAME,
    required=True,
    type=WOW_ADDONS_PATH_TYPE,
    help=(
        'Your WoW addons path, such as "/path/to/WoW/Interface/Addons". May also be '
        "specified in the environment variable WAP_WOW_ADDONS_PATH."
    ),
)
def dev_install(
    config_path: Path,
    addon_version: str,
    output_path: Path,
    wow_addons_path: Path,
    show_json: bool,
) -> None:
    config = Config.from_path(config_path)

    wow_addons_path_type = WoWVersion.addons_path_type(wow_addons_path.parts[-3])

    wow_version: Optional[WoWVersion] = None
    for wow_version_in_config in config.wow_versions:
        if wow_version_in_config.type() == wow_addons_path_type:
            wow_version = wow_version_in_config
            break
    else:
        raise DevInstallException(
            f"No build exists for WoW addons path {wow_addons_path} (which is a "
            f"{wow_addons_path_type} installation). "
        )

    output_map = {}

    build_path = addon.build_addon(
        config_path=config_path,
        addon_name=config.name,
        dir_configs=config.dir_configs,
        output_path=output_path,
        addon_version=addon_version,
        wow_version=wow_version,
    )

    dev_install_paths = addon.dev_install_addon(
        build_path=build_path,
        wow_addons_path=wow_addons_path,
        addon_name=config.name,
    )

    output_map[wow_version.type()] = {
        "build_dir_path": str(build_path),
        "installed_dir_paths": [str(path) for path in dev_install_paths],
    }

    if show_json:
        click.echo(json.dumps(output_map, indent=2))



from pathlib import Path

import click

from wap.commands.common import DEFAULT_CONFIG_PATH, PATH_TYPE
from wap.config import default_config
from wap.exception import QuickstartException
from wap.log import info
from wap.wowversion import WoWVersion

LATEST_RETAIL_VERSION = WoWVersion.from_dot_version("9.0.2")


def write_changelog(path: Path, name: str) -> None:
    with path.open("w") as changelog_file:
        changelog_file.write(f"# {name} Changelog\n\n")
        changelog_file.write(
            "This file is used by *wap* when you upload to Curseforge. It should "
            "contain a record of changes over time to your project.\n\n"
        )
        changelog_file.write("## Example Version 0.0.1\n\n")
        changelog_file.write("- Added feature X\n")
        changelog_file.write("- Fixed bug Y\n")


def write_readme(path: Path, name: str) -> None:
    with path.open("w") as readme_file:
        readme_file.write(f"# {name}\n\n")
        readme_file.write(
            "(This file is not required to run *wap*. It is here as a suggestion to "
            "document your project.)\n\n"
        )
        readme_file.write(
            'This file is written in Markdown, "a lightweight and easy-to-use syntax '
            'for styling all forms of writing".\n\n'
        )
        readme_file.write(
            "You can learn more about Markdown at "
            "<https://guides.github.com/features/mastering-markdown/>.\n\n"
        )
        readme_file.write(
            "Otherwise, go ahead and describe your project, such as...\n\n"
        )
        readme_file.write("## Features\n\n")
        readme_file.write("- Gives you free gold!\n")
        readme_file.write("- Boosts your DPS!\n")
        readme_file.write(
            "- Did someone say Thunderfury, Blessed Blade of the Windseeker?\n"
        )


def write_lua_file(path: Path, name: str) -> None:
    with path.open("w") as lua_file:
        lua_file.write("-- Your code can go here.\n")
        lua_file.write("-- Here's something to get you started,\n")
        lua_file.write("-- but you can erase it if you wish.\n\n")
        lua_file.write(f'local title = GetAddOnMetadata("{name}", "Title")\n')
        lua_file.write(f'local version = GetAddOnMetadata("{name}", "Version")\n')
        lua_file.write(f'print(title .. " version " .. version .. " initialized")\n')


@click.command()
@click.argument(
    "project_dir_path",
    type=PATH_TYPE,
)
def quickstart(
    project_dir_path: Path,
) -> None:
    """
    Creates a new addon project directory at PROJECT_DIR_PATH with a default structure.
    PROJECT_DIR_PATH must not exist. The final path component of PROJECT_DIR_PATH will
    be used as the addon's name, the name of the directory in `dirs`, and the Lua file
    created. These are merely suggestions to get you started and may be changed at will.
    """
    if project_dir_path.exists():
        raise QuickstartException(
            f"{project_dir_path} exists. Choose a path that does not exist for your "
            "new project."
        )

    info(f"Creating project directory at {project_dir_path}")
    project_dir_path.mkdir(parents=True)

    project_name = project_dir_path.name

    config = default_config(project_name)
    config_path = project_dir_path / DEFAULT_CONFIG_PATH
    info(f"Creating config file at {config_path}")
    config.to_path(config_path)

    changelog_path = project_dir_path / "CHANGELOG.md"
    info(f"Creating changelog file at {changelog_path}")
    write_changelog(changelog_path, project_name)

    readme_path = project_dir_path / "README.md"
    info(f"Creating readme at {readme_path}")
    write_readme(readme_path, project_name)

    lua_file = project_dir_path / project_name / (project_name + ".lua")
    info(f"Creating starter lua file at {lua_file}")
    lua_file.parent.mkdir()
    write_lua_file(lua_file, project_name)

    info(f"Project created! You can now begin developing your project.")
    info(
        f"After you `cd {project_dir_path}`, you can get started running some wap "
        "commands immediately, such as:"
    )
    info("  - wap build")
    info(
        R"  - wap dev-install --wow-addons-path"
        R'"C:\Program Files (x86)\World of Warcraft\_retail_\Interface\AddOns"'
    )
    info("Have fun!")



from pathlib import Path

import click

from wap.commands.common import DEFAULT_CONFIG_PATH, PATH_TYPE
from wap.config import default_config
from wap.exception import NewConfigException
from wap.log import info


@click.command()
@click.option(
    "-c",
    "--config-path",
    type=PATH_TYPE,
    default=str(DEFAULT_CONFIG_PATH),
    show_default=str(DEFAULT_CONFIG_PATH),
    help=("The path of the configuration file."),
)
def new_config(
    config_path: Path,
) -> None:
    """
    Creates a new configuration file with some pre-filled data.

    The name of the current directory will be used as the name of the addon and as the
    configured directory and lua code file: These are merely suggestions to get you
    started and may be changed at will.
    """
    if config_path.exists():
        raise NewConfigException(f"{config_path} exists. Aborting to avoid data loss.")

    project_name = Path.cwd().name

    config = default_config(project_name)
    info(f"Creating config file at {config_path}")
    config.to_path(config_path)



import shutil
from pathlib import Path


def delete_path(path: Path) -> None:
    """
    Deletes either a file or directory. The path existance must be checked before
    calling this function.
    """
    if path.is_file():
        path.unlink()
    elif path.is_dir():
        shutil.rmtree(path)
    else:  # pragma: no cover
        # don't measure code coverage here because it's a really esoteric case:
        # it could be a pipe or socket or some other non-file thing.
        raise ValueError(
            f"Cannot delete path {path} because it is not a file or directory"
        )



import shutil
from collections.abc import Sequence
from pathlib import Path
from zipfile import ZIP_DEFLATED, ZipFile

from wap import log
from wap.config import CurseforgeConfig, DirConfig
from wap.curseforge import CHANGELOG_SUFFIX_MAP, CurseForgeAPI
from wap.exception import BuildException, UploadException
from wap.toc import Toc
from wap.util import delete_path
from wap.wowversion import WoWVersion


def build_addon(
    config_path: Path,
    addon_name: str,
    dir_configs: Sequence[DirConfig],
    output_path: Path,
    addon_version: str,
    wow_version: WoWVersion,
) -> Path:
    build_path = output_path / f"{addon_name}-{addon_version}-{wow_version.type()}"

    if build_path.exists():
        delete_path(build_path)

    build_path.mkdir(parents=True)

    for dir_config in dir_configs:
        src_dir = config_path.parent / dir_config.path

        if not src_dir.is_dir():
            raise BuildException(
                f"{src_dir} is not a directory",
            )

        dst_dir = build_path / src_dir.name

        shutil.copytree(src_dir, dst_dir)

        toc_path = dst_dir / (src_dir.name + ".toc")

        if toc_path.exists():
            log.warn(
                f"A TOC file at {toc_path.name} exists in {dir_config.path}, but will "
                "be overwritten."
            )
            delete_path(toc_path)

        toc_config = dir_config.toc_config
        toc = Toc.from_toc_config(toc_config)
        toc.write(
            path=toc_path,
            addon_version=addon_version,
            wow_version=wow_version,
        )

    log.info(f"Built addon {addon_name} ({wow_version.type()}) at {build_path}")

    return build_path


def zip_addon(addon_name: str, wow_version: WoWVersion, build_path: Path) -> Path:
    zip_path = build_path.parent / (build_path.name + ".zip")

    if zip_path.exists():
        delete_path(zip_path)

    zip_path.parent.mkdir(parents=True, exist_ok=True)

    with ZipFile(zip_path, mode="w", compression=ZIP_DEFLATED) as zip_file:
        for path in build_path.rglob("*"):
            zip_file.write(
                filename=path,
                arcname=path.relative_to(build_path),
            )

    log.info(f"Zipped addon {addon_name} ({wow_version.type()}) at {zip_path}")

    return zip_path


def upload_addon(
    *,
    addon_name: str,
    config_path: Path,
    curseforge_config: CurseforgeConfig,
    wow_version: WoWVersion,
    zip_file_path: Path,
    addon_version: str,
    release_type: str,
    curseforge_api: CurseForgeAPI,
) -> str:
    cf_wow_version_id = curseforge_api.get_version_id(version=wow_version.dot_version())

    changelog_path = config_path.parent / curseforge_config.changelog_path
    if not changelog_path.is_file():
        raise UploadException(f"Changelog file {changelog_path} is not a file")

    with changelog_path.open("r") as changelog_file:
        changelog_contents = changelog_file.read()

    normalized_changelog_suffix = changelog_path.suffix.lower()
    if normalized_changelog_suffix in CHANGELOG_SUFFIX_MAP:
        changelog_type = CHANGELOG_SUFFIX_MAP[normalized_changelog_suffix]
    else:
        log.warn(
            f"Unable to determine changelog type from extension for {changelog_path}, "
            'so assuming "text"'
        )
        changelog_type = "text"

    with zip_file_path.open("rb") as package_archive_file:
        file_id = curseforge_api.upload_addon_file(
            project_id=curseforge_config.project_id,
            archive_file=package_archive_file,
            display_name=f"{addon_version}-{wow_version.type()}",
            changelog_contents=changelog_contents,
            changelog_type=changelog_type,
            wow_version_id=cf_wow_version_id,
            release_type=release_type,
            file_name=f"{addon_name}-{addon_version}-{wow_version.type()}.zip",
        )

    url = curseforge_api.uploaded_file_url(
        addon_name=curseforge_config.addon_name,
        file_id=file_id,
    )
    log.info(f"Uploaded {addon_name} to CurseForge at {url}")

    return url


def dev_install_addon(
    *,
    build_path: Path,
    addon_name: str,
    wow_addons_path: Path,
) -> Sequence[Path]:
    installed_paths = []

    for addon_path in build_path.iterdir():
        install_addon_path = wow_addons_path / addon_path.name

        if install_addon_path.exists():
            delete_path(install_addon_path)

        shutil.copytree(addon_path, install_addon_path)

        installed_paths.append(install_addon_path)

    log.info(f"Installed addon {addon_name} to {wow_addons_path}")
    return installed_paths



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['wap', 'wap.commands']

package_data = \
{'': ['*']}

install_requires = \
['arrow>=0.17.0,<0.18.0',
 'attrs>=20.3.0,<21.0.0',
 'click>=7.1.2,<8.0.0',
 'requests>=2.25.1,<3.0.0',
 'strictyaml>=1.3.2,<2.0.0']

entry_points = \
{'console_scripts': ['wap = wap.__main__:main']}

setup_kwargs = {
    'name': 'wow-addon-packager',
    'version': '0.4.1',
    'description': 'A tool that builds your World of Warcraft addon and uploads them to CurseForge',
    'long_description': None,
    'author': 'Tim Martin',
    'author_email': 'tim@timmart.in',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from __future__ import annotations

import json
from pathlib import Path

import click

from wap import addon, log
from wap.commands.common import (
    DEFAULT_RELEASE_TYPE,
    WAP_CURSEFORGE_TOKEN_ENVVAR_NAME,
    addon_version_option,
    config_path_option,
    json_option,
    output_path_option,
)
from wap.config import Config
from wap.curseforge import CurseForgeAPI
from wap.exception import UploadException


@click.command()
@config_path_option()
@output_path_option()
@json_option()
@addon_version_option(required=True)
@click.option(
    "-r",
    "--release-type",
    type=click.Choice(list(CurseForgeAPI.RELEASE_TYPES)),
    default=DEFAULT_RELEASE_TYPE,
    show_default=True,
    help="The type of release to make.",
)
@click.option(
    "--curseforge-token",
    envvar=WAP_CURSEFORGE_TOKEN_ENVVAR_NAME,
    required=True,
    help=(
        "The value of your CurseForge API token. May also be specified in the "
        "environment variable WAP_CURSEFORGE_TOKEN."
    ),
)
def upload(
    config_path: Path,
    output_path: Path,
    addon_version: str,
    release_type: str,
    curseforge_token: str,
    show_json: bool,
) -> None:
    """
    Uploads packages to CurseForge.
    """
    config = Config.from_path(config_path)

    curseforge_config = config.curseforge_config
    if curseforge_config is None:
        raise UploadException(
            'A "curseforge" configuration section must be provided in config to upload'
        )

    output_map = {}

    curseforge_api = CurseForgeAPI(api_token=curseforge_token)

    for wow_version in config.wow_versions:
        build_path = addon.build_addon(
            config_path=config_path,
            addon_name=config.name,
            dir_configs=config.dir_configs,
            output_path=output_path,
            addon_version=addon_version,
            wow_version=wow_version,
        )

        zip_path = addon.zip_addon(
            addon_name=config.name,
            wow_version=wow_version,
            build_path=build_path,
        )

        upload_url = addon.upload_addon(
            addon_name=config.name,
            curseforge_config=curseforge_config,
            config_path=config_path,
            wow_version=wow_version,
            zip_file_path=zip_path,
            addon_version=addon_version,
            release_type=release_type,
            curseforge_api=curseforge_api,
        )

        output_map[wow_version.type()] = {
            "build_dir_path": str(build_path),
            "zip_file_path": str(zip_path),
            "curseforge_upload_url": upload_url,
        }

    if show_json:
        click.echo(json.dumps(output_map, indent=2))
